//
// HackerBox 0104 Demo code for NodeMCU ESP8266 OLED Board
//    to trigger display modes on Alien Robot Badge
//
// Transmitted mode is randomly selected 0..19
// First three args are randomly selected 0..255 (usually RGB colors)
//
// ESP-NOW wireless functionality inspired by Rui Santos' example:
// https://RandomNerdTutorials.com/esp-now-esp8266-nodemcu-arduino-ide/
// 

#include <Arduino.h>
#include <Wire.h>
#include <U8g2lib.h>
#include <ESP8266WiFi.h>
#include <espnow.h>

#define flash_button_pin 0

U8G2_SSD1306_128X64_NONAME_F_SW_I2C u8g2(U8G2_R0,12,14,U8X8_PIN_NONE);

uint8_t broadcastAddress[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

typedef struct pkt_struct {
    uint32_t tag;
    uint8_t display_mode;
    uint8_t arg[10];
} struct_message;

pkt_struct rx_packet;
pkt_struct tx_packet;

unsigned long packets_tx = 0;  
unsigned long packets_rx = 0;
uint8_t last_mode_tx = 99;
uint8_t last_mode_rx = 99;

void PacketSent(uint8_t *mac_addr, uint8_t sendStatus) {
  Serial.print("Last Packet Send Status: ");
  if (sendStatus == 0){
    Serial.println("Delivery success");
  }
  else{
    Serial.println("Delivery fail");
  }
}

void PacketReceived(uint8_t * mac, uint8_t *incomingData, uint8_t len) {
  memcpy(&rx_packet, incomingData, sizeof(rx_packet));
  if(rx_packet.tag == 0x13374AC5) { //check tag to verify it's one of ours
    packets_rx++;
    last_mode_rx = rx_packet.display_mode;
    update_display();
  }
}

void update_display(){ 
  char buff[64];
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_7x14B_tr);
  sprintf(buff, "Packets RXed: %d", packets_rx);
  u8g2.drawStr(0, 15, buff);
  sprintf(buff, "Last RX Mode: %d", last_mode_rx);
  u8g2.drawStr(0, 30, buff);
  sprintf(buff, "Packets TXed: %d", packets_tx);
  u8g2.drawStr(0, 45, buff);
  sprintf(buff, "Last TX Mode: %d", last_mode_tx);
  u8g2.drawStr(0, 60, buff);
  u8g2.sendBuffer();
}

void setup() {
  pinMode(flash_button_pin, INPUT_PULLUP);
  u8g2.begin();
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);

  if (esp_now_init() != 0) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  esp_now_set_self_role(ESP_NOW_ROLE_COMBO);
  esp_now_register_recv_cb(PacketReceived);
  esp_now_register_send_cb(PacketSent);

  update_display();
}

void loop() {
  if(digitalRead(flash_button_pin) == 0) {
    tx_packet.tag = 0x13374AC5;  //elite hacks :)
    tx_packet.display_mode = random(20);
    tx_packet.arg[0] = random(255);
    tx_packet.arg[1] = random(255);
    tx_packet.arg[2] = random(255);

    // Send message via ESP-NOW
    esp_now_send(broadcastAddress, (uint8_t *) &tx_packet, sizeof(tx_packet));

    packets_tx++;
    last_mode_tx = tx_packet.display_mode;
    update_display();
    delay(300); // debounce button
  }
}